@@ -1,5 +1,4 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Harness.pm,v 1.76 2003/11/25 04:41:03 andy Exp $
 
 package Test::Harness;
 
@@ -29,13 +28,11 @@ Test::Harness - Run Perl standard test scripts with statistics
 
 =head1 VERSION
 
-Version 2.38
-
-    $Header: /home/cvs/test-harness/lib/Test/Harness.pm,v 1.76 2003/11/25 04:41:03 andy Exp $
+Version 2.44
 
 =cut
 
-$VERSION = '2.38';
+$VERSION = "2.44";
 
 # Backwards compatibility for exportable variable names.
 *verbose  = *Verbose;
@@ -463,15 +460,12 @@ sub _run_all_tests {
                 bench    => 0,
                );
 
-    my @dir_files = _globdir $Files_In_Dir if defined $Files_In_Dir;
+    my @dir_files;
+    @dir_files = _globdir $Files_In_Dir if defined $Files_In_Dir;
     my $t_start = new Benchmark;
 
     my $width = _leader_width(@tests);
     foreach my $tfile (@tests) {
-	if ( $Test::Harness::Debug ) {
-	    print "# Running: ", $Strap->_command_line($tfile), "\n";
-	}
-
         $Last_ML_Print = 0;  # so each test prints at least once
         my($leader, $ml) = _mk_leader($tfile, $width);
         local $ML = $ml;
@@ -481,6 +475,9 @@ sub _run_all_tests {
         $tot{files}++;
 
         $Strap->{_seen_header} = 0;
+        if ( $Test::Harness::Debug ) {
+            print "# Running: ", $Strap->_command_line($tfile), "\n";
+        }
         my %results = $Strap->analyze_file($tfile) or
           do { warn $Strap->{error}, "\n";  next };
 
@@ -548,7 +545,7 @@ sub _run_all_tests {
             }
             elsif($results{seen}) {
                 if (@{$test{failed}} and $test{max}) {
-                    my ($txt, $canon) = canonfailed($test{max},$test{skipped},
+                    my ($txt, $canon) = _canonfailed($test{max},$test{skipped},
                                                     @{$test{failed}});
                     print "$test{ml}$txt";
                     $failedtests{$tfile} = { canon   => $canon,
@@ -597,7 +594,7 @@ sub _run_all_tests {
                 @dir_files = @new_dir_files;
             }
         }
-    }
+    } # foreach test
     $tot{bench} = timediff(new Benchmark, $t_start);
 
     $Strap->_restore_PERL5LIB;
@@ -699,8 +696,14 @@ sub _show_results {
 }
 
 
-my %Handlers = ();
-$Strap->{callback} = sub {
+my %Handlers = (
+    header => \&header_handler,
+    test => \&test_handler,
+    bailout => \&bailout_handler,
+);
+
+$Strap->{callback} = \&strap_callback;
+sub strap_callback {
     my($self, $line, $type, $totals) = @_;
     print $line if $Verbose;
 
@@ -709,7 +712,7 @@ $Strap->{callback} = sub {
 };
 
 
-$Handlers{header} = sub {
+sub header_handler {
     my($self, $line, $type, $totals) = @_;
 
     warn "Test header seen more than once!\n" if $self->{_seen_header};
@@ -721,7 +724,7 @@ $Handlers{header} = sub {
          $totals->{max}  < $totals->{seen};
 };
 
-$Handlers{test} = sub {
+sub test_handler {
     my($self, $line, $type, $totals) = @_;
 
     my $curr = $totals->{seen};
@@ -753,7 +756,7 @@ $Handlers{test} = sub {
 
 };
 
-$Handlers{bailout} = sub {
+sub bailout_handler {
     my($self, $line, $type, $totals) = @_;
 
     die "FAILED--Further testing stopped" .
@@ -811,7 +814,7 @@ sub _dubious_return {
            $wstatus,$wstatus;
     print "\t\t(VMS status is $estatus)\n" if $^O eq 'VMS';
 
-    if (corestatus($wstatus)) { # until we have a wait module
+    if (_corestatus($wstatus)) { # until we have a wait module
         if ($Have_Devel_Corestack) {
             Devel::CoreStack::stack($^X);
         } else {
@@ -830,7 +833,7 @@ sub _dubious_return {
         else {
             push @{$test->{failed}}, $test->{'next'}..$test->{max};
             $failed = @{$test->{failed}};
-            (my $txt, $canon) = canonfailed($test->{max},$test->{skipped},@{$test->{failed}});
+            (my $txt, $canon) = _canonfailed($test->{max},$test->{skipped},@{$test->{failed}});
             $percent = 100*(scalar @{$test->{failed}})/$test->{max};
             print "DIED. ",$txt;
         }
@@ -900,7 +903,7 @@ sub _create_fmts {
 {
     my $tried_devel_corestack;
 
-    sub corestatus {
+    sub _corestatus {
         my($st) = @_;
 
         my $did_core;
@@ -920,7 +923,7 @@ sub _create_fmts {
     }
 }
 
-sub canonfailed ($$@) {
+sub _canonfailed ($$@) {
     my($max,$skipped,@failed) = @_;
     my %seen;
     @failed = sort {$a <=> $b} grep !$seen{$_}++, @failed;
@@ -957,8 +960,8 @@ sub canonfailed ($$@) {
 	push @result, "?% okay";
     }
     my $ender = 's' x ($skipped > 1);
-    my $good = $max - $failed - $skipped;
     if ($skipped) {
+        my $good = $max - $failed - $skipped;
 	my $skipmsg = " (less $skipped skipped test$ender: $good okay, ";
 	if ($max) {
 	    my $goodper = sprintf("%.2f",100*($good/$max));
@@ -1123,6 +1126,7 @@ Here's how Test::Harness tests itself
 
 =head1 SEE ALSO
 
+The included F<prove> utility for running test scripts from the command line,
 L<Test> and L<Test::Simple> for writing test scripts, L<Benchmark> for
 the underlying timing routines, L<Devel::CoreStack> to generate core
 dumps from failed tests and L<Devel::Cover> for test coverage
