@@ -2,8 +2,7 @@ package Test::Harness::Iterator;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = 0.01;
-
+$VERSION = 0.02;
 
 =head1 NAME
 
@@ -23,6 +22,14 @@ B<FOR INTERNAL USE ONLY!>
 
 This is a simple iterator wrapper for arrays and filehandles.
 
+=head2 new()
+
+Create an iterator.
+
+=head2 next()
+
+Iterate through it, of course.
+
 =cut
 
 sub new {
@@ -47,8 +54,7 @@ sub new {
 
 package Test::Harness::Iterator::FH;
 sub next {
-    my $fh = $_[0]->{fh};
-    return scalar <$fh>;
+    return scalar readline( $_[0]->{fh} );
 }
 
 
